package client.gui;

import java.security.KeyPair;
import java.security.PublicKey;

import client.gui.GuiConnect.ServerInfo;
import client.gui.element.ActButton;
import client.gui.element.ButtonCallback;
import client.gui.element.Label;
import client.gui.element.NavButton;
import client.gui.element.PressType;
import client.gui.element.Toggle;
import client.gui.element.FieldAction;
import client.gui.element.Field;
import client.gui.element.FieldCallback;
import client.vars.CVarCategory;
import client.vars.Variable;
import common.color.TextColor;
import common.network.IPlayer;
import common.util.EncryptUtil;

public class GuiServer extends Gui implements FieldCallback {
	public static final GuiServer INSTANCE = new GuiServer(null);
	
	private final ServerInfo server;

	private Field nameBox;
	private Field addrBox;
	private Field portBox;
	private Field userBox;
	private Field passBox;
	private Field accBox;
	private Label nameLabel;
	private Label addrLabel;
	private Label portLabel;
	private Label rangeLabel;
	private Label userLabel;
	private Label passLabel;
	private Label accLabel;
	private Toggle encToggle;
	private ActButton keyButton;
	private ActButton resetButton;
	private KeyPair keypair;
	private PublicKey serverKey;
	
	public GuiServer(ServerInfo server) {
		this.server = server;
	}

    @Variable(name = "srv_last_address", category = CVarCategory.SYSTEM, min = 1, max = 128, display = "Letzte Server-Adresse")
	private String lastAddr = "";
	@Variable(name = "srv_last_port", category = CVarCategory.SYSTEM, min = 1024, max = 32767, display = "Letzter Server-Port")
	private int lastPort = -1;
    @Variable(name = "srv_last_user", category = CVarCategory.SYSTEM, max = IPlayer.MAX_USER_LENGTH, display = "Letzter Server-Nutzer", validator = IPlayer.UserValidator.class)
	private String lastUser = "";
    @Variable(name = "srv_last_password", category = CVarCategory.SYSTEM, max = IPlayer.MAX_PASS_LENGTH, display = "Letztes Server-Passwort")
	private String lastPass = "";
    @Variable(name = "srv_last_access", category = CVarCategory.SYSTEM, max = IPlayer.MAX_PASS_LENGTH, display = "Letzter Server-Zugang")
	private String lastAcc = "";
	
	public void init(int width, int height) {
		if(this.server != null)
			this.nameBox = this.add(new Field(0, -50, 400, 24, 128, this, this.server.getName()));
		this.addrBox = this.add(new Field(0, 20, 400, 24, 128, this, this.server == null ? this.lastAddr : this.server.getAddress()));
		int port = this.server == null ? this.lastPort : this.server.getPort();
		this.portBox = this.add(new Field(404, 20, 76, 24, 5, this, port < 0 ? "" : "" + port));
		this.userBox = this.add(new Field(0, 70, 220, 24, IPlayer.MAX_USER_LENGTH, this, IPlayer.VALID_USER, this.server == null ? this.lastUser : this.server.getUser()));
		this.passBox = this.add(new Field(0, 120, 480, 24, IPlayer.MAX_PASS_LENGTH, this, this.server == null ? this.lastPass : this.server.getPassword()));
		this.accBox = this.add(new Field(0, 170, 480, 24, IPlayer.MAX_PASS_LENGTH, this, this.server == null ? this.lastAcc : this.server.getAccess()));
		this.add(new ActButton(0, this.server == null ? 220 : 370, 480, 24, new ButtonCallback() {
			public void use(ActButton elem, PressType action) {
				GuiServer.this.connect();
			}
		}, this.server == null ? "Verbinden" : (this.server.getName().isEmpty() ? "Hinzufügen" : "Übernehmen")));
		this.add(new NavButton(0, this.server == null ? 250 : 400, 480, 24, this.server != null ? GuiConnect.INSTANCE : GuiMenu.INSTANCE, "Zurück"));
		if(this.server != null)
			this.nameLabel = this.add(new Label(0, -70, 410, 20, "Name", true));
		this.addrLabel = this.add(new Label(0, 0, 410, 20, "Adresse", true));
		this.portLabel = this.add(new Label(404, 0, 76, 20, "Port", true));
		this.rangeLabel = this.add(new Label(370, 44, 110, 20, "[1024..32767]", true));
		this.userLabel = this.add(new Label(0, 50, 220, 20, "Nutzer", true));
		this.passLabel = this.add(new Label(0, 100, 480, 20, "Passwort (mind. 8 Zeichen)", true));
		this.accLabel = this.add(new Label(0, 150, 480, 20, "Zugang (mind. 8 Zeichen)", true));
		if(this.server != null) {
			this.keyButton = this.add(new ActButton(0, 220, 480, 24, new ButtonCallback() {
				public void use(ActButton elem, PressType action) {
					if(GuiServer.this.keypair == null) {
						GuiServer.this.keypair = EncryptUtil.generateKeyPair();
						GuiServer.this.keyButton.setText("Schlüsselpaar entfernen");
					}
					else {
						final String name = GuiServer.this.nameBox.getText();
						final String addr = GuiServer.this.addrBox.getText();
						final String port = GuiServer.this.portBox.getText();
						final String pass = GuiServer.this.passBox.getText();
						final String acc = GuiServer.this.accBox.getText();
						final boolean reqEnc = GuiServer.this.encToggle.getValue();
						final KeyPair keys = GuiServer.this.keypair;
						final PublicKey key = GuiServer.this.serverKey;
						GuiServer.this.gm.displayGuiScreen(new GuiConfirm(new GuiConfirm.Callback() {
							public void confirm(boolean confirmed) {
								GuiServer.this.gm.displayGuiScreen(GuiServer.this);
								GuiServer.this.nameBox.setText(name);
								GuiServer.this.addrBox.setText(addr);
								GuiServer.this.portBox.setText(port);
								GuiServer.this.passBox.setText(pass);
								GuiServer.this.accBox.setText(acc);
								GuiServer.this.encToggle.setValue(reqEnc);
								GuiServer.this.serverKey = key;
								GuiServer.this.resetButton.enabled = key != null;
								if(confirmed) {
									GuiServer.this.keypair = null;
									GuiServer.this.keyButton.setText("Neues Schlüsselpaar generieren");
								}
								else {
									GuiServer.this.keypair = keys;
									GuiServer.this.keyButton.setText("Schlüsselpaar entfernen");
								}
							}
						}, "Schlüsselpaar wirklich entfernen?", "Wenn das Schlüsselpaar gelöscht wird, ist es nicht mehr möglich, sich damit auf dem Server zu identifizieren und sich anzumelden.\nDamit könnte der Zugriff auf den Server unmöglich werden.", "Schlüsselpaar löschen", "Abbrechen"));
					}
				}
			}, (this.keypair = this.server.getKeypair()) == null ? "Neues Schlüsselpaar generieren" : "Schlüsselpaar entfernen"));
			this.encToggle = this.add(new Toggle(0, 270, 480, 24, false, this.server.requiresEncryption(), null, "Nur Verschlüsselt verbinden"));
			GuiServer.this.serverKey = this.server.getServerKey();
			this.resetButton = this.add(new ActButton(0, 320, 480, 24, new ButtonCallback() {
				public void use(ActButton elem, PressType action) {
					if(GuiServer.this.serverKey != null) {
						GuiServer.this.serverKey = null;
						GuiServer.this.resetButton.enabled = false;
					}
				}
			}, "Server-Key zurücksetzen"));
			this.resetButton.enabled = this.serverKey != null;
		}
		this.shift();
	}
	
	public String getTitle() {
		return this.server == null ? "Mit Server verbinden" : (this.server.getName().isEmpty() ? "Server hinzufügen" : "Server bearbeiten");
	}
	
	private void connect() {
		if(this.gm.world != null)
			return;
		String name = null;
		if(this.server != null) {
			name = this.nameBox.getText();
			if(name.isEmpty()) {
				this.nameLabel.setText(TextColor.RED + "Name");
				return;
			}
		}
		String addr = this.addrBox.getText();
		if(addr.isEmpty()) {
			this.addrLabel.setText(TextColor.RED + "Adresse");
			return;
		}
		int port = -1;
		if(this.portBox.getText().isEmpty()) {
			this.portLabel.setText(TextColor.RED + "Port");
			return;
		}
		else {
			try {
				port = Integer.parseInt(this.portBox.getText());
			}
			catch(NumberFormatException e) {
				this.rangeLabel.setText("[" + TextColor.RED + "1024..32767" + TextColor.RESET + "]");
				return;
			}
			if(port < 1024 || port > 32767) {
				this.rangeLabel.setText("[" + (port < 1024 ? TextColor.RED + "1024" + TextColor.RESET + "..32767" : "1024.." + TextColor.RED + "32767" + TextColor.RESET) + "]");
				return;
			}
		}
		String user = this.userBox.getText();
		if(user.isEmpty()) {
			this.userLabel.setText(TextColor.RED + "Nutzer");
			return;
		}
		String pass = this.passBox.getText();
		if(pass.length() < 8) {
			this.passLabel.setText(pass.isEmpty() ? TextColor.RED + "Passwort (mind. 8 Zeichen)" : "Passwort (" + TextColor.RED + "mind. 8 Zeichen" + TextColor.RESET + ")");
			return;
		}
		String acc = this.accBox.getText();
		if(acc.length() < 8) {
			this.accLabel.setText(acc.isEmpty() ? TextColor.RED + "Zugang (mind. 8 Zeichen)" : "Zugang (" + TextColor.RED + "mind. 8 Zeichen" + TextColor.RESET + ")");
			return;
		}
		if(this.server == null) {
			this.lastAddr = addr;
			this.lastPort = port;
			this.lastUser = user;
			this.lastPass = pass;
			this.lastAcc = acc;
			this.gm.setDirty();
			GuiConnect.INSTANCE.connect(addr, port, user, pass, acc);
		}
		else {
			this.server.setData(name, addr, port, user, pass, acc, this.keypair, this.serverKey, this.encToggle.getValue());
			GuiConnect.INSTANCE.applyServer(this.server);
		}
	}
	
	public void use(Field elem, FieldAction value) {
		if(value == FieldAction.SEND) {
			elem.setDeselected();
			this.connect();
		}
		else if(value == FieldAction.FOCUS) {
			if(elem == this.addrBox)
				this.addrLabel.setText("Adresse");
			else if(elem == this.portBox) {
				this.portLabel.setText("Port");
				this.rangeLabel.setText("[1024..32767]");
			}
			else if(elem == this.userBox)
				this.userLabel.setText("Nutzer");
			else if(elem == this.passBox)
				this.passLabel.setText("Passwort (mind. 8 Zeichen)");
			else if(elem == this.accBox)
				this.accLabel.setText("Zugang (mind. 8 Zeichen)");
			else if(this.server != null && elem == this.nameBox)
				this.nameLabel.setText("Name");
		}
	}
}
